<?php
namespace App\Controllers;

use \Psr\Http\Message\ServerRequestInterface as Request;
use \Psr\Http\Message\ResponseInterface as Response;
use \Psr\Http\Message\UploadedFileInterface as UploadedFile;

use App\Models\CasoClinico;
use App\Models\Producto;
use App\Models\DocumentoCasoClinico;
use Carbon\Carbon;

class CasosClinicosController
{
    public function ReadAll(Request $request, Response $response, $args)
    {
      $casosClinicos = CasoClinico::all();
      $casosClinicos->loadMissing(['productos', 'seguimientos', 'documentos', 'medico', 'enfermera', 'estado']);
      return $response->getBody()->write(json_encode($casosClinicos));
    }
    public function Create(Request $request, Response $response, $args)
    {
        $headers = $request->getHeaders();

        $casoClinicoNuevo = $request->getParsedBody();
       
        $casoClinico = CasoClinico::create($casoClinicoNuevo);

        $casoClinico->loadMissing(['productos', 'enfermera', 'estado', 'centromedico', 'institucion', 'documentos', 'seguimientos']);

        //$this->Correo_CasoClinico($casoClinico);

        return $response->getBody()->write(json_encode($casoClinico));
    }
    public function Read(Request $request, Response $response, $args)
    {
      $id = $request->getAttribute('id');

      $casoClinico = CasoClinico::find($id);
      $casoClinico->loadMissing(['productos', 'seguimientos', 'documentos', 'medico', 'enfermera', 'estado']);

      return $response->getBody()->write(json_encode($casoClinico));
    }
    public function Update(Request $request, Response $response, $args)
    {
        $id = $request->getAttribute('id');
        $casoClinicoUpdate = $request->getParsedBody();

        $casoClinico = CasoClinico::find($id);
        $casoClinico->update($casoClinicoUpdate);

        $casoClinico->loadMissing(['productos', 'enfermera', 'estado', 'centromedico', 'institucion', 'documentos', 'seguimientos']);

        return $response->getBody()->write(json_encode($casoClinico));
    }
    public function Delete(Request $request, Response $response, $args)
    {
      $id = $request->getAttribute('id');

      $casoClinico = CasoClinico::find($id);
      $casoClinico->delete();

      return $response->getBody()->write(json_encode($casoClinico));
    }
    public function Folio(Request $request, Response $response, $args)
    {
      $id = $request->getAttribute('id');

      $casoClinico = CasoClinico::all()->last();
      // OLD
      //$noCasosClinicosMedico = CasoClinico::where("fk_medico", $id)
      //->whereYear('fechaRegistro', getdate()["year"])->count();

      $noCasosClinicosMedico = CasoClinico::where("fk_medico", $id)
      ->whereYear('created_at', getdate()["year"])->count();
      $newRow = [
                "serie" => "CC" . str_pad(($casoClinico ? $casoClinico->id : 0) + 1, 5, "0", STR_PAD_LEFT),
                "folio" => "CX" . str_pad($noCasosClinicosMedico + 1, 4, "0", STR_PAD_LEFT) . '-' . substr(getdate()["year"], 2, 2)
                ];

      return $response->getBody()->write(json_encode($newRow));
    }
    public function UpCasoClinicoEnfermeraProPuntos(Request $request, Response $response, $args)
    {
       $id = $request->getAttribute('id');
       $enfermera = $request->getParsedBody();

       $fk_enfermera = ['fk_enfermera' =>  $enfermera['id'] ];
       $casoclinico = CasoClinico::find($id);
       $casoclinico->update($fk_enfermera);

       $fk_enfermera_puntos = ['fk_enfermera' => $enfermera['id'], 'e_puntos' => $enfermera['puntos']];

       $noAfectados = Producto::where('fk_casoClinico', $id)->update($fk_enfermera_puntos);

       $casoclinico->loadMissing(['productos.enfermera']);

       return $response->getBody()->write(json_encode($casoclinico));
    }
	public function Query(Request $request, Response $response, $args)
    {
      //$id = $request->getAttribute('id');
        $casoClinico = $request->getParsedBody();
        $casoClinicoFind = [];

        foreach ($casoClinico as $i => $value) {
                if(empty($value)){
                    unset($casoClinico[$i]);
                }
        }

        if(empty($casoClinico)){
          $casoClinicoFind = CasoClinico::all();
          $casoClinicoFind->loadMissing(['productos', 'seguimientos', 'documentos', 'medico', 'enfermera', 'estado']);
        }
        else {
          $casoClinicoFind = CasoClinico::where($casoClinico)->get();
          $casoClinicoFind->loadMissing(['productos', 'seguimientos', 'documentos', 'medico', 'enfermera', 'estado']);
        }

        return $response->getBody()->write(json_encode($casoClinicoFind));
    }
    public function Fechas(Request $request, Response $response, $args)
    {

      $id = $request->getAttribute('id');

      $fechaAplicacionProducto = null;
      $fechaPrimerSeguimiento = null;
      $dias = '0';
      $activo = false;
      $fechaHoy = Carbon::now();
      $minFecha = Carbon::now();
      $maxFecha = Carbon::now();
      $casoClinico = CasoClinico::find($id);

      if($casoClinico)
      {

        $fechaAplicacionProducto = Carbon::create($casoClinico->fechaAplicacionProducto);

        $fechaPrimerSeguimiento = Carbon::create($casoClinico->fechaAplicacionProducto);
        $fechaPrimerSeguimiento->addWeeks(5);

        $dias = $fechaHoy->diffInDays($fechaPrimerSeguimiento);

        $activo = $fechaPrimerSeguimiento > $fechaHoy;
        $minFecha->add(0, 'day');
        $maxFecha->add(3, 'day');
      } 
	  else
	  {
        $minFecha->add(-15, 'day');
        $maxFecha->add(0, 'day');
      }

      $fechas = [
                 'fechaAplicacionProducto' => $fechaAplicacionProducto,
                 'fechaPrimerSeguimiento' => $fechaPrimerSeguimiento,
                 'fechaHoy' => $fechaHoy,
                 'fechaMin' => $minFecha,
                 'fechaMax' => $maxFecha,
                 'dias' => $dias,
                 'activo' => $activo
               ];

      return $response->getBody()->write(json_encode($fechas));
    }
    public function UpFile(Request $request, Response $response, $args)
    {
      $res = [
              "code" => 0,
              "message" => "No se cumplio la funcion",
              "data" => null
            ];

      $fk_casoClinico   = $request->getParam('fk_casoClinico');
      $fk_tipoDocumento = $request->getParam('fk_tipoDocumento');
      $descDocumento    = $request->getParam('descDocumento');
    
      $seccion          = $request->getParam('seccion');
      $noSeguimiento    = $request->getParam('noSeguimiento');

      $files = $request->getUploadedFiles();
      $file = $files['file'];

      if ($file->getError() === UPLOAD_ERR_OK)
      {

        $path = $_SERVER['DOCUMENT_ROOT'] . '/img/';

        $media = $file->getClientMediaType();

        $extension = pathinfo($file->getClientFilename(), PATHINFO_EXTENSION);
        $basename = 'cc' . $fk_casoClinico . $fk_tipoDocumento . strval(mt_rand());
        $filename = sprintf('%s.%0.8s', $basename, $extension);
        $size = $file->getSize();

        $file->moveTo( $path . $filename );

        $ruta = '/img/' . $filename;

        $documento = new DocumentoCasoClinico();

        $documento->fk_casoClinico = $fk_casoClinico;
        $documento->fk_tipoDocumento = $fk_tipoDocumento;
        $documento->descDocumento = $descDocumento;
        $documento->nomDocumento = $filename;

        $documento->ruta = $ruta;
        $documento->media = $media;
        $documento->extension = $extension;
        $documento->tamano = $size;
       // $documento->fechaDocumento = $fechaDocumento;
       // $documento->fechaRegistro =  Carbon::now();
        $documento->seccion  = $seccion;
        $documento->noSeguimiento = $noSeguimiento;

        if($documento->save())
        {
          $res["code"] = 1;
          $res["message"] = "Registrado con exito";
          $res["data"] = $documento;
        }
        else
        {
          $res["code"] = 0;
          $res["message"] = "No se registrado";
          $res["data"] = $documento;
        }
      }
      return $response->getBody()->write(json_encode($res));
    }
    public function DeleteFile(Request $request, Response $response, $args)
    {
		$res = [
				  "code" => 0,
				  "message" => "No se cumplio la funcion",
				  "data" => null
				];

		$id = $request->getAttribute('id');

		$referencia = DocumentoCasoClinico::find($id);
		$documento = $referencia;

		$ruta = dirname(__DIR__, 3) . DIRECTORY_SEPARATOR . '/img/' . DIRECTORY_SEPARATOR . $referencia->nomDocumento;
		
		if(file_exists($ruta)) {
			
			if(unlink($ruta)) {
				$referencia->delete();
				$res["code"] = 1;
				$res["message"] = 'El documento se elimino satisfactoriamente!';
				$res["data"] = $documento;
			  }else {
				  $res["code"] = 0;
				  $res["message"] = 'There was a error deleting the file ' . $referencia->nomDocumento;
				  $res["data"] = $documento;
			  }
		
		}else {
				  $res["code"] = 0;
				  $res["message"] = 'Archivo ' . $ruta  . ' no fue localizado';
				  $res["data"] = $documento;

		}

		return $response->getBody()->write(json_encode($res));
    }
	public function uploadFile(Request $request, Response $response, $args)
    {

      $files = $request->getUploadedFiles(); //array UploadedFile

      $file = $files['imagen'];

      $media = $file->getClientMediaType();
      $size = $file->getSize();
      $name = $file->getClientFilename();
      $error = $file->getError();
      $extension = pathinfo($file->getClientFilename(), PATHINFO_EXTENSION);
      //echo   $media, ' ', $size, ' ', $name, ' ', $error, ' ', $extension;
      if($file->getError() === UPLOAD_ERR_OK)
      {
          $this->moveUploadFile($file);
      }
     //echo var_dump($files);
    }
    private function moveUploadFile(UploadedFile $upLoadedFile)
    {
          $path = $_SERVER['DOCUMENT_ROOT'] . '/img/';
          $upLoadedFile->moveTo($path . $upLoadedFile->getClientFilename());
          return $upLoadedFile->getClientFilename();
    }
    private function Correo_CasoClinico($casoClinico)
    {

      $medico = $casoClinico->medico ? $casoClinico->medico->nombre . ' ' . $casoClinico->medico->primerApellido : '';
      $folio = $casoClinico->folio;
      $serie = $casoClinico->serie;
      $paciente = $casoClinico->paciente;
      $hospital = $casoClinico->hospital;
      $diagnostico = $casoClinico->diagnostico;

      $enfermera = $casoClinico->enfermera ?  $casoClinico->enfermera->nombre . ' ' . $casoClinico->enfermera->primerApellido : '';
      $fechaCC = "----";
      $fecha = $casoClinico->fecha;
      if($fecha) {
        $date = date_create($fecha);
        $fechaCC = date_format($date,"d/m/Y");
      }

      $para_medico = $casoClinico->medico->correo;
      $para_alertas = 'alertas_cc@biotechsa.com';
      $titulo = 'ALTA DE EXPEDIENTE';
            // mensaje
      $mensaje_medico = "
                        <html>
                          <head>
                            <title>ALTA DE EXPEDIENTE</title>
                          </head>
                          <body>
                            <div class='contenedor'>
                            <div>
                              <img src='https://casosclinicos.online/imgmail/altaexpediente.png'/>
                            </div>
                            <p>Estimado Dr. <span> $medico </span></p>
                            <p>Nos complace informar que el caso clinico con el No. CX  $folio  se ha registrado de manera exitosa. Te invitamos a dar seguimiento a tu expediente e incrementar tus puntos en cada uno.</p>
                            <p>Saludos</p>
                            <p><a href='https://casosclinicos.online'>www.casosclinicos.online</a></p>
                            </div>
                          </body>
                        </html>";
                  // mensaje
      $mensaje_alertas = "
                          <html>
                                <head>
                                  <title>ALTA DE EXPEDIENTE (Caso Clínico)</title>
                                </head>
                                <body>
                                  <div class='contenedor'>
                                    <div>
                                    <img src='https://casosclinicos.online/imgmail/altaexpediente.png'/>
                                    </div>
									<h1> CORREO DE PRUEBA  DATOS NO VALIDOS </h1>
									
                                    <p>Fecha Alta: $fechaCC </p>
                                    <p>Médico Tratante: $medico </p>
                                    <p>Alta Caso Clínico Consecutivo: $serie </p>
                                    <p>Alta Caso Clínico Médico: $folio </p>
                                    <p>Nombre Paciente: $paciente </p>
                                    <p>Hospital: $hospital </p>
                                    <p>Diagnostico: $diagnostico </p>
                                    <p>Técnico: $enfermera </p>
                                    <p> <a href='https://casosclinicos.online'>www.casosclinicos.online</a> </p>
                                  </div>
                                </body>
                          </html>
                        ";


                  // Para enviar un correo HTML, debe establecerse la cabecera Content-type
            $cabeceras  = 'From: alertas@casosclinicos.online' . "\r\n" .
                          'Reply-To: alertas@casosclinicos.online' . "\r\n" .
                          'X-Mailer: PHP/' . " \r\n";
            $cabeceras .= 'MIME-Version: 1.0' . "\r\n";
            $cabeceras .= 'Content-type: text/html; charset=utf-8' . "\r\n";

            // Enviarlo charset=iso-8859-1
            mail($para_alertas, $titulo, $mensaje_alertas, $cabeceras);

            // Enviarlo
           mail($para_medico, $titulo, $mensaje_medico, $cabeceras);
    }
}
