<?php
namespace App\Controllers;

use \Psr\Http\Message\ServerRequestInterface as Request;
use \Psr\Http\Message\ResponseInterface as Response;

use App\Models\Usuario;
use App\Models\Medico;
use App\Models\Enfermera;

use App\Models\RedimirEnfermera;
use App\Models\Producto;
use App\Models\CasoClinico;
use App\Models\RedimirEnfermeraReferencia;
use Carbon\Carbon;

class EnfermerasController
{
  public function ReadAll(Request $request, Response $response, $args)
  {
    $enfermeras = Enfermera::all();
    $enfermeras->loadMissing(['usuario', 'casosclinicos', 'medicos', 'redimirenfermera', 'productos', 'supervisor']);

    return $response->getBody()->write(json_encode($enfermeras));
  }

  public function Create(Request $request, Response $response, $args)
  {
    $data = $request->getParsedBody();
    $usuario = $data["usuario"];
    $enfermera = $data["enfermera"];

    $enfermera_nueva = new Enfermera($enfermera);

    $usuario_nuevo = Usuario::create($usuario);
    $usuario_nuevo->enfermera()->save($enfermera_nueva);

    return $response->getBody()->write(json_encode($enfermera_nueva));
  }

  public function Read(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id');
    $enfermera = Enfermera::find($id);

    $enfermera->loadMissing(['supervisor']);
    $enfermera->loadMissing(['redimirenfermera.referencias']);
    $enfermera->loadMissing(['usuario.rol', 'usuario.documentos']);
    $enfermera->loadMissing(['casosclinicos.productos', 'casosclinicos.medico', 'casosclinicos.estado', 'casosclinicos.centromedico', 'casosclinicos.institucion', 'casosclinicos.documentos', 'casosclinicos.seguimientos']);
    $enfermera->loadMissing(['medicos.casosclinicos', 'medicos.productos', 'medicos.redimirmedico']);
    $enfermera->loadMissing(['productos.casoclinico', 'productos.medico']);

    return $response->getBody()->write(json_encode($enfermera));
  }

  public function Update(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id');
    $enfermeraUpdate = $request->getParsedBody();

    $enfermera = Enfermera::find($id);
    $enfermera->update($enfermeraUpdate);

    return $response->getBody()->write(json_encode($enfermera));
  }

  public function Delete(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id', '0');
    $enfermera = Enfermera::find($id);
    $enfermera->delete();

    return $response->getBody()->write(json_encode($enfermera));
  }

  public function Totales(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id');

    $enfermera = Enfermera::find($id);

    $enfermera->loadMissing(['casosclinicos',  'redimirenfermera', 'productos', 'supervisor']);

    $casosclinicos = $enfermera->casosclinicos->count();
    $productos = $enfermera->productos->count();
    $puntos = $enfermera->productos->sum('puntos');
    $canjes = $enfermera->redimirenfermera->where('status', 3)->sum('redimir');

    $saldo = $puntos - $canjes;
    $totales = ['casosclinicos' => $casosclinicos, 'productos' => $productos, 'puntos' => $puntos, 'saldo' => $saldo, 'redimirenfermera'=> $canjes];

    return $response->getBody()->write(json_encode($totales));
  }

  public function RelacionCasosClinicos(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id');

    $casosclinicos = CasoClinico::where('fk_enfermera', $id)->orderBy('fecha', 'desc')->get();

    return $response->getBody()->write(json_encode($casosclinicos));
  }

  public function RelacionProductos(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id');

    $enfermera = Enfermera::find($id);

    return $response->getBody()->write(json_encode($enfermera->productos));
  }

  public function Puntos(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id');

    $informacionPuntos = Producto::select(
                        'Productos.id',
                        'Productos.e_puntos',
                        'Productos.fechaAplicacion',
                        'CasosClinicos.folio',
                        'CasosClinicos.serie',
                        'Productos.nomProducto',
                        'Productos.codigo')->where('Productos.fk_enfermera', $id)
                      ->leftjoin('CasosClinicos','Productos.fk_casoClinico','=', 'CasosClinicos.id')
                      ->orderBy('Productos.fechaAplicacion', 'desc')
                      ->get();

    return $response->getBody()->write(json_encode($informacionPuntos));
  }

  public function Canjes(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id');

    $canjes = RedimirEnfermera::where('fk_enfermera', $id)->orderBy('fechaInicio', 'desc')->get();

    return $response->getBody()->write(json_encode($canjes));
  }

  public function EstadoCuenta(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id');

    $canjes = RedimirEnfermera::where('fk_enfermera', $id)->where('status', 3)
             ->select('id', 'fechaFinal as fecha', 'redimir as canje')
             ->selectRaw("null as folio")
             ->selectRaw('0 as puntos')
             ->selectRaw('1 as cantidad')
             ->selectRaw("'Canje' as concepto");

    $puntos = Producto::where('fk_enfermera', $id)
              ->select('fk_casoClinico')
              ->selectRaw('0 as canje')
              ->selectRaw("'Cirugia' as concepto")
              ->selectRaw('sum(e_puntos) as puntos')
              ->selectRaw('count(e_puntos) as cantidad')
              ->groupBy('fk_casoClinico');

    $sorted = CasoClinico::select(
               'CasosClinicos.id',
               'CasosClinicos.fechaAplicacionProducto as fecha',
               'puntos.canje',
               'CasosClinicos.folio',
               'puntos.puntos',
               'puntos.cantidad',
               'puntos.concepto')
               ->joinSub($puntos, 'puntos', function ($j) {
                $j->on("CasosClinicos.id", "=", "puntos.fk_casoClinico" );
               })->where('CasosClinicos.fk_enfermera', $id)->union($canjes)->orderBy('fecha', 'desc')->get();

    return $response->getBody()->write(json_encode($sorted));
  }

  public function AsociarMedico(Request $request, Response $response, $args)
  {
      $res = ["code" => 0, "message" => "" , "data" => null];

      $id = $request->getAttribute('id');
      $id_medico = $request->getParam('id');

      $enfermera = Enfermera::find($id);
      $enfermera->medicos()->attach($id_medico);

      $medico = Medico::find($id_medico);

      $res["code"] = 1;
      $res["message"] = "Asocidado correctamente";
      $res["data"] = $medico;


      return $response->getBody()->write(json_encode($res));
  }
  
  public function DesasociarMedico(Request $request, Response $response, $args)
  {
    $res = ["code" => 0, "message" => "" , "data" => null];

      $id = $request->getAttribute('id');
      $id_medico = $request->getParam('id');

      $enfermera = Enfermera::find($id);
      $n = $enfermera->medicos()->detach($id_medico);

      $medico = Medico::find($id_medico);
      if($n){
        $res["code"] = 1;
        $res["message"] = "Registro de baja con exito";
        $res["data"] = $medico;
      }else{
        $res["code"] = 0;
        $res["message"] = "No se registro";
        $res["data"] = $medico;
      }

      return $response->getBody()->write(json_encode($res));
  }

  public function RedimirCreate(Request $request, Response $response, $args)
  {
    $redimirNuevo = $request->getParsedBody();
    
    $redimir = RedimirEnfermera::create($redimirNuevo);
    $id = $redimir->fk_enfermera;
    $enfermera = Enfermera::find($id);

    $enfermera->loadMissing(['redimirenfermera', 'productos']);

    //$this->correo_solicitud_canje($enfermera, $redimir);

    return $response->getBody()->write(json_encode($redimir));
  }

  public function RedimirUpdate(Request $request, Response $response, $args)
  {

    //	$redimirMedico = $request->getParsedBody(); see http://php.net/manual/en/function.random-bytes.php 
		$redimirEnfermera = null;
		
		$id = $request->getParam('id');
		$redimir = $request->getParam('redimir');
		$fk_enfermera = $request->getParam('fk_enfermera');
		$fechaFinal = $request->getParam('fechaFinal');
		$observacion = $request->getParam('observacion');
		$status = $request->getParam('status');

		$files = $request->getUploadedFiles();
		$file = $files['file'];

		if ($file->getError() === UPLOAD_ERR_OK) {

			$path = $_SERVER['DOCUMENT_ROOT'] . '/referencia/enfermeras/';

			$media = $file->getClientMediaType();

			$extension = pathinfo($file->getClientFilename(), PATHINFO_EXTENSION);
			$basename = 'referencia' . $fk_enfermera . '-' . strval(mt_rand());
			$filename = sprintf('%s.%0.8s', $basename, $extension);
			$size = $file->getSize();

			$file->moveTo( $path . $filename );

			$ruta = '/referencia/enfermeras/' . $filename;
		
			$redimirEnfermeraReferencia = new RedimirEnfermeraReferencia([	
																	"ruta" => $ruta,
																	"media" => $media,
																	"extension" => $extension,
																	"filename"  => $filename,
																	"size" => $size
																	]);	
			$redimirEnfermera = RedimirEnfermera::find($id);

			$redimirEnfermera->redimir = $redimir;
			$redimirEnfermera->fechaFinal = $fechaFinal;
			$redimirEnfermera->observacion = $observacion;
			$redimirEnfermera->status = $status;
		
			$redimirEnfermera->referencias->save($redimirEnfermeraReferencia);
		
			$redimirEnfermera->save();
			$redimirEnfermera->loadMissing(['referencias']);
      }
	  
      return $response->getBody()->write(json_encode($redimirEnfermera));
  }
  
  public function RedimirUpdateRef(Request $request, Response $response, $args)
  {
      // see http://php.net/manual/en/function.random-bytes.php
      // $redimirMedico = $request->getParsedBody();
      $res = ["code" => 0, "message" => "No se cumplio la funcion" , "data" => null];
      $id = $request->getParam('id');
      $fk_enfermera = $request->getParam('fk_enfermera');
      $status = $request->getParam('status');

      $files = $request->getUploadedFiles();
      $file = $files['file'];

      if ($file->getError() === UPLOAD_ERR_OK) {

        $path = $_SERVER['DOCUMENT_ROOT'] . '/referencia/enfermeras/';

        $media = $file->getClientMediaType();

        $extension = pathinfo($file->getClientFilename(), PATHINFO_EXTENSION);
        $basename = 'referencia' . $fk_enfermera . '-' . strval(mt_rand());
        $filename = sprintf('%s.%0.8s', $basename, $extension);
        $size = $file->getSize();

        $file->moveTo( $path . $filename );

        $ruta = '/referencia/enfermeras/' . $filename;

        $redimirEnfermera = RedimirEnfermera::find($id);

        $redimirEnfermera->ruta = $ruta;
        $redimirEnfermera->media = $media;
        $redimirEnfermera->extension = $extension;
        $redimirEnfermera->filename  = $filename;
        $redimirEnfermera->size = $size;

        if($redimirEnfermera->save()){
          $res["code"] = 1;
          $res["message"] = "Actualizado con exito";
          $res["data"] = $redimirEnfermera;
        }else{
          $res["code"] = 0;
          $res["message"] = "No se Actualizo";
          $res["data"] = $redimirEnfermera;
        }
      }
      return $response->getBody()->write(json_encode($res));
  }

  public function RedimirUpdateRechazado(Request $request, Response $response, $args)
  {
      $id = $request->getParam('id');
      $fechaFinal = $request->getParam('fechaFinal');
      $observacion = $request->getParam('observacion');
      $status = $request->getParam('status');

      $redimirEnfermera = RedimirEnfermera::find($id);
      $redimirEnfermera->fechaFinal = $fechaFinal;
      $redimirEnfermera->observacion = $observacion;
      $redimirEnfermera->status = $status;
      $redimirEnfermera->save();

      return $response->getBody()->write(json_encode($redimirEnfermera));
  }
 
  public function RedimirDelete(Request $request, Response $response, $args)
  {
      $res = [
              "code" => 0,
              "message" => "No se cumplio la funcion",
              "data" => null
            ];

      $id_enfermera = $request->getAttribute('idE');
      $id_canje  = $request->getAttribute('idC');

      $redimirEnfermera = RedimirEnfermera::find($id_canje);
      $redimirEnfermera->delete();
      $res["code"] = 1;
      $res["message"] = 'Canje eliminado correctamente!';
      $res["data"] = $redimirEnfermera;

      return $response->getBody()->write(json_encode($res));
  }
  /* NUEVA FUNCION */
  public function RedimirAddReferencia(Request $request, Response $response, $args)
  {	
		
		$fk_redimir = $request->getParam('fk_redimir');

		$files = $request->getUploadedFiles();
		$file = $files['file'];

		if ($file->getError() === UPLOAD_ERR_OK) {

			$path = $_SERVER['DOCUMENT_ROOT'] . '/referencia/enfermeras/';

			$media = $file->getClientMediaType();

			$extension = pathinfo($file->getClientFilename(), PATHINFO_EXTENSION);
			$basename = 'referencia' . $fk_redimir . '-' . strval(mt_rand());
			$filename = sprintf('%s.%0.8s', $basename, $extension);
			$size = $file->getSize();

			$file->moveTo( $path . $filename );

			$ruta = '/referencia/enfermeras/' . $filename;

			$redimirEnfermeraReferencia = RedimirEnfermeraReferencia::create([
																		"fk_redimir" => $fk_redimir, 
																		"ruta" => $ruta, 
																		"media" => $media,
																		"extension" => $extension, 
																		"filename" => $filename, 
																		"size" => $size
																		]);
			 
		}
		return $response->getBody()->write(json_encode($redimirEnfermeraReferencia));
	}
  
  public function DeleteFileReferencia(Request $request, Response $response, $args)
  {
		$res = [
				  "code" => 0,
				  "message" => "No se cumplio la funcion",
				  "data" => null
				];

		$id = $request->getAttribute('id');

		$referencia = RedimirEnfermeraReferencia::find($id);
		$referencia->ruta;
		$documento = $referencia;

		$ruta = dirname(__DIR__, 3) . DIRECTORY_SEPARATOR . 'referencia\enfermeras' . DIRECTORY_SEPARATOR . $referencia->filename;
		
		if(file_exists($ruta)) {
			
			if(unlink($ruta)) {
				$referencia->delete();
				$res["code"] = 1;
				$res["message"] = 'El documento se elimino satisfactoriamente!';
				$res["data"] = $documento;
			  }else {
				  $res["code"] = 0;
				  $res["message"] = 'There was a error deleting the file ' . $referencia->nomDocumento;
				  $res["data"] = $documento;
			  }
		
		}else {
				  $res["code"] = 0;
				  $res["message"] = 'Archivo ' . $ruta  . ' no fue localizado';
				  $res["data"] = $documento;

		}

		return $response->getBody()->write(json_encode($res));
  }
  
  private function correo_solicitud_canje($enfermera, $redimir)
  {
        $nomCompleto = ($enfermera->nombre ?  $enfermera->nombre . " " . $enfermera->primerApellido : "");

        $saldo = 0;
        $redimirTotal = 0;
        $puntosTotal = 0;

        foreach($enfermera->redimirenfermera as $rd) {
                if($rd->status == 3)
                   $redimirTotal += $rd->redimir;
        }
        foreach($enfermera->productos as $producto) {
                   $puntosTotal += $producto->e_puntos;
        }
        $saldo = $puntosTotal - $redimirTotal;

        $importe = $redimir->redimir;
        $descripcion = $redimir->observacion;
        $fechaSolicitud = "----";
        $fecha = $redimir->fechaInicio;

        if($fecha) {
          $date = date_create($fecha);
          $fechaSolicitud = date_format($date,"d/m/Y");
        }
        $mensaje_enfermera = "
                              <html>
                                <head>
                                    <title>SOLICITUD CANJE (Caso Clínico)</title>
                                </head>
                                <body>
                                    <div class='contenedor'>
                                      <div><img src='https://casosclinicos.online/imgmail/redimirpuntos.png'/></div>
                                        <p>Estimado(a)  <span>  $nomCompleto </span>, hemos recibido su petición de canje de sus puntos de casosclinicos.online</p>


                                        <p> Saludos. </p>
                                      <a href='https://casosclinicos.online'>www.casosclinicos.online</a>
                                    </div>
                                </body>
                                </html>";

        $mensaje_alertas = "
                            <html>
                            <head>
                              <title>SOLICITUD CANJE DE ENFERMERA/TECNICO (Caso Clínico)</title>
                            </head>
                            <body>
                            <div class='contenedor'>
                                  <div><img src='https://casosclinicos.online/imgmail/congreso.png'/></div>
									<h1> CORREO DE PRUEBA  DATOS NO VALIDOS </h1>		
                                  <p>Fecha Solicitud: $fechaSolicitud</p>
                                  <p>Nombre Técnico: $nomCompleto</p>
                                  <p>Importe Solicitando: $importe</p>
                                  <p>Saldo: $saldo</p>
                                  <p>Comentario: $descripcion</p>
                                  <p><a href='https://casosclinicos.online'>www.casosclinicos.online</a></p>
                            </div>
                            </body>
                            </html>
                            ";
        $para_enfermera = ($enfermera ? $enfermera->correo : 'alertas_cc@biotechsa.com');

        $para_alertas = 'alertas_cc@biotechsa.com';
        $titulo = "SOLICITUD CANJE TECNICO";
        // Para enviar un correo HTML, debe establecerse la cabecera Content-type
        $cabeceras  = 'From: alertas@casosclinicos.online' . "\r\n" .
        'Reply-To: alertas@casosclinicos.online' . "\r\n" .
        'X-Mailer: PHP/' . " \r\n";
        $cabeceras .= 'MIME-Version: 1.0' . "\r\n";
        $cabeceras .= 'Content-type: text/html; charset=utf-8' . "\r\n";

        // Enviarlo
       mail($para_alertas, $titulo, $mensaje_alertas, $cabeceras);
        // Enviarlo
       mail($para_enfermera, $titulo, $mensaje_enfermera, $cabeceras);
  }

}
