<?php
namespace App\Controllers;

use \Psr\Http\Message\ServerRequestInterface as Request;
use \Psr\Http\Message\ResponseInterface as Response;

use App\Models\Usuario;
use App\Models\Medico;
use App\Models\Enfermera;

use App\Models\RedimirMedico;
use App\Models\Producto;
use App\Models\CasoClinico;

use App\Models\RedimirMedicoReferencia;
use Carbon\Carbon;

class MedicosController
{

	public function ReadAll(Request $request, Response $response, $args)
	{
		$medicos = Medico::all();
		$medicos->loadMissing([ 'casosclinicos', 'enfermeras', 'redimirmedico', 'productos', 'usuario']);

		return $response->getBody()->write(json_encode($medicos));
    }
	public function Create(Request $request, Response $response, $args)
	{
		$medico_nuevo = $request->getParsedBody();

		$medico = Medico::create($medico_nuevo);

		return $response->getBody()->write(json_encode($medico));
	}
	public function Read(Request $request, Response $response, $args)
	{
		$id = $request->getAttribute('id');
		$medico = Medico::find($id);

		$medico->loadMissing(['puesto', 'residencia']);
		$medico->loadMissing(['redimirmedico.referencias']);
		$medico->loadMissing(['usuario.rol', 'usuario.documentos']);
		$medico->loadMissing(['casosclinicos.productos', 'casosclinicos.enfermera', 'casosclinicos.estado', 'casosclinicos.centromedico', 'casosclinicos.institucion', 'casosclinicos.documentos', 'casosclinicos.seguimientos']);
		$medico->loadMissing(['enfermeras.casosclinicos', 'enfermeras.productos', 'enfermeras.redimirenfermera']);
		$medico->loadMissing(['productos.casoclinico', 'productos.enfermera', 'productos.medico']);
		
		return $response->getBody()->write(json_encode($medico));
    }
	public function Update(Request $request, Response $response, $args)
	{
		$id = $request->getAttribute('id', '0');
		$medicoUpdate = $request->getParsedBody();

		$medico = Medico::find($id);
		$medico->update($medicoUpdate);

		return $response->getBody()->write(json_encode($medico));
	}
	public function Delete(Request $request, Response $response, $args)
	{
		$id = $request->getAttribute('id');

		$medico = Medico::find($id);
		$medico->Delete();

		return $response->getBody()->write(json_encode($medico));
	}
	public function RelacionEnfermeras(Request $request, Response $response, $args)
	{
		$id = $request->getAttribute('id');

		$medico = Medico::find($id);
		$medico->loadMissing(['enfermeras']);

		return $response->getBody()->write(json_encode($medico));
	}
	public function RelacionCasosClinicos(Request $request, Response $response, $args)
	{
		$id = $request->getAttribute('id');

		$medico = Medico::find($id);
		$medico->loadMissing(['casosclinicos']);

		return $response->getBody()->write(json_encode($medico->casosclinicos));
	}
	public function RelacionProductos(Request $request, Response $response, $args)
	{
		$id = $request->getAttribute('id');

		$medico = Medico::find($id);
		$medico->loadMissing(['productos']);

		return $response->getBody()->write(json_encode($medico->productos));
	}
	public function Puntos(Request $request, Response $response, $args)
	{
		$id = $request->getAttribute('id');

		$informacionPuntos = Producto::select(
							'Productos.fk_producto',
							'Productos.m_puntos',
							'Productos.fechaAplicacion',
							'CasosClinicos.folio',
							'CasosClinicos.serie',
							'Productos.nomProducto',
							'Productos.codigo')->where('Productos.fk_medico', $id)
						  ->leftjoin('CasosClinicos','Productos.fk_casoClinico', '=', 'CasosClinicos.id')
						  ->orderBy('Productos.fechaAplicacion', 'desc')
						  ->get();

		return $response->getBody()->write(json_encode($informacionPuntos));
	}
	public function EstadoCuenta(Request $request, Response $response, $args)
	{
    $id = $request->getAttribute('id');

    $canjes = RedimirMedico::where('fk_medico', $id)->where('status', 3)
             ->select('id', 'fechaFinal as fecha', 'redimir as canje')
             ->selectRaw("null as folio")
             ->selectRaw('0 as puntos')
             ->selectRaw('1 as cantidad')
             ->selectRaw("'Canje' as concepto");

    $puntos = Producto::where('fk_medico', $id)
              ->select('fk_casoClinico')
              ->selectRaw('0 as canje')
              ->selectRaw("'Cirugia' as concepto")
              ->selectRaw('sum(m_puntos) as puntos')
              ->selectRaw('count(m_puntos) as cantidad')
              ->groupBy('fk_casoClinico');

    $sorted = CasoClinico::select(
               'CasosClinicos.id',
               'CasosClinicos.fechaAplicacionProducto as fecha',
               'puntos.canje',
               'CasosClinicos.folio',
               'puntos.puntos',
               'puntos.cantidad',
               'puntos.concepto')
               ->joinSub($puntos, 'puntos', function ($j) {
                $j->on("CasosClinicos.id", "=", "puntos.fk_casoClinico" );
               })->where('CasosClinicos.fk_medico', $id)->union($canjes)->orderBy('fecha', 'desc')->get();

    return $response->getBody()->write(json_encode($sorted));
  }
	public function Canjes(Request $request, Response $response, $args)
	{
    $id = $request->getAttribute('id');

    $canjes = RedimirMedico::where('fk_medico', $id)->orderBy('fechaInicio', 'desc')->get();

    return $response->getBody()->write(json_encode($canjes));
    }
	public function RedimirCreate(Request $request, Response $response, $args)
	{
		$redimirNuevo = $request->getParsedBody();	   

		$redimir = RedimirMedico::create($redimirNuevo);

		$id = $redimir->fk_medico; 
		
		$medico = Medico::find($id);
		$medico->loadMissing(['redimirmedico', 'productos']);

		$this->correo_solicitud_canje($medico, $redimir);

		return $response->getBody()->write(json_encode($redimir));
	}
	public function RedimirUpdate(Request $request, Response $response, $args)
	{
		$redimirMedico = null;
		
		$id = $request->getParam('id');
		$redimir = $request->getParam('redimir');
		$fk_medico = $request->getParam('fk_medico');
		$fechaFinal = $request->getParam('fechaFinal');
		$observacion = $request->getParam('observacion');
		$status = $request->getParam('status');

		$files = $request->getUploadedFiles();
		$file = $files['file'];

		if ($file->getError() === UPLOAD_ERR_OK) {

			$path = $_SERVER['DOCUMENT_ROOT'] . '/referencia/medicos/';

			$media = $file->getClientMediaType();

			$extension = pathinfo($file->getClientFilename(), PATHINFO_EXTENSION);
			$basename = 'referencia' . $fk_medico . '-' . strval(mt_rand());
			$filename = sprintf('%s.%0.8s', $basename, $extension);
			$size = $file->getSize();

			$file->moveTo( $path . $filename );

			$ruta = '/referencia/medicos/' . $filename;
		
			$redimirMedicoReferencia = new RedimirMedicoReferencia(["ruta" => $ruta,
																	"media" => $media,
																	"extension" => $extension,
																	"filename"  => $filename,
																	"size" => $size]);
			$redimirMedico = RedimirMedico::find($id);

			$redimirMedico->redimir = $redimir;
			$redimirMedico->fechaFinal = $fechaFinal;
			$redimirMedico->observacion = $observacion;
			$redimirMedico->status = $status;        
																
			$redimirMedico->referencias()->save($redimirMedicoReferencia);														
		
			$redimirMedico->save();
			$redimirMedico->loadMissing(['referencias']);
		}
		return $response->getBody()->write(json_encode($redimirMedico));
    }
	public function RedimirRefUpdate(Request $request, Response $response, $args)
	{
    $res = ["code" => 0, "message" => "No se cumplio la funcion" , "data" => null];

    $id = $request->getParam('id');
    $fk_medico = $request->getParam('fk_medico');
    $status = $request->getParam('status');

    $files = $request->getUploadedFiles();
    $file = $files['file'];

    if ($file->getError() === UPLOAD_ERR_OK) {

		$path = $_SERVER['DOCUMENT_ROOT'] . '/referencia/medicos/';

		$media = $file->getClientMediaType();

		$extension = pathinfo($file->getClientFilename(), PATHINFO_EXTENSION);
		$basename = 'referencia' . $fk_medico . '-' . strval(mt_rand());
		$filename = sprintf('%s.%0.8s', $basename, $extension);
		$size = $file->getSize();

		$file->moveTo( $path . $filename );

		$ruta = '/referencia/medicos/' . $filename;

		$redimirMedico = RedimirMedico::find($id);

		$redimirMedico->ruta = $ruta;
		$redimirMedico->media = $media;
		$redimirMedico->extension = $extension;
		$redimirMedico->filename  = $filename;
		$redimirMedico->size = $size;

      if($redimirMedico->save()){
        $res['code'] = 1;
        $res['message'] = ' con exito';
        $res['data'] = $redimirMedico;
      }else{
        $res['code'] = 0;
        $res['message'] = 'No se registrado';
        $res['data'] = $redimirMedico;
      }
    }
    return $response->getBody()->write(json_encode($res));
  }
	public function RedimirUpdateRef(Request $request, Response $response, $args)
	{
		$res = ["code" => 0, "message" => "No se cumplio la funcion" , "data" => null];

		$id = $request->getParam('id');
		$fk_medico = $request->getParam('fk_medico');
		$status = $request->getParam('status');

		$files = $request->getUploadedFiles();
		$file = $files['file'];

		if ($file->getError() === UPLOAD_ERR_OK) {

		  $path = $_SERVER['DOCUMENT_ROOT'] . '/referencia/medicos/';

		  $media = $file->getClientMediaType();

		  $extension = pathinfo($file->getClientFilename(), PATHINFO_EXTENSION);
		  $basename = 'referencia' . $fk_medico . '-' . strval(mt_rand());
		  $filename = sprintf('%s.%0.8s', $basename, $extension);
		  $size = $file->getSize();

		  $file->moveTo( $path . $filename );

		  $ruta = '/referencia/medicos/' . $filename;

		  $redimirMedico = RedimirMedico::find($id);

		  $redimirMedico->ruta = $ruta;
		  $redimirMedico->media = $media;
		  $redimirMedico->extension = $extension;
		  $redimirMedico->filename  = $filename;
		  $redimirMedico->size = $size;

		  if($redimirMedico->save()){
			$res['code'] = 1;
			$res['message'] = 'Actualizado con exito';
			$res['data'] = $redimirMedico;
		  }else{
			$res['code'] = 0;
			$res['message'] = 'No se Actualizado';
			$res['data'] = $redimirMedico;
		  }
		}
		return $response->getBody()->write(json_encode($res));
	}
	public function RedimirUpdateRechazado(Request $request, Response $response, $args)
	{
		$id = $request->getParam('id');
		$fechaFinal = $request->getParam('fechaFinal');
		$observacion = $request->getParam('observacion');
		$status = $request->getParam('status');

		$redimirMedico = RedimirMedico::find($id);
		$redimirMedico->fechaFinal = $fechaFinal;
		$redimirMedico->observacion = $observacion;
		$redimirMedico->status = $status;
		$redimirMedico->save();

		return $response->getBody()->write(json_encode($redimirMedico));
	}
	public function RedimirDelete(Request $request, Response $response, $args)
	{
		$res = [
              "code" => 0,
              "message" => "No se cumplio la funcion",
              "data" => null
            ];
		$id_medico = $request->getAttribute('idM');
		$id_canje  = $request->getAttribute('idC');

		$redimirMedico = RedimirMedico::find($id_canje);
		$redimirMedico->delete();
		$res["code"] = 1;
		$res["message"] = 'Canje eliminado correctamente!!';
		$res["data"] = $redimirMedico;

		return $response->getBody()->write(json_encode($res));
	}
	/* NUEVA FUNCION */
	public function RedimirAddReferencia(Request $request, Response $response, $args)
	{	
		
		$fk_redimir = $request->getParam('fk_redimir');

		$files = $request->getUploadedFiles();
		$file = $files['file'];

		if ($file->getError() === UPLOAD_ERR_OK) {

			$path = $_SERVER['DOCUMENT_ROOT'] . '/referencia/medicos/';

			$media = $file->getClientMediaType();

			$extension = pathinfo($file->getClientFilename(), PATHINFO_EXTENSION);
			$basename = 'referencia' . $fk_redimir . '-' . strval(mt_rand());
			$filename = sprintf('%s.%0.8s', $basename, $extension);
			$size = $file->getSize();

			$file->moveTo( $path . $filename );

			$ruta = '/referencia/medicos/' . $filename;

			$redimirMedicoReferencia = RedimirMedicoReferencia::create([
																		"fk_redimir" => $fk_redimir, 
																		"ruta" => $ruta, 
																		"media" => $media,
																		"extension" => $extension, 
																		"filename" => $filename, 
																		"size" => $size
																		]);
			 
		}
		return $response->getBody()->write(json_encode($redimirMedicoReferencia));
	}
	public function DeleteFileReferencia(Request $request, Response $response, $args)
    {
		$res = [
				  "code" => 0,
				  "message" => "No se cumplio la funcion",
				  "data" => null
				];

		$id = $request->getAttribute('id');

		$referencia = RedimirMedicoReferencia::find($id);
		$referencia->ruta;
		$documento = $referencia;

		$ruta = dirname(__DIR__, 3) . DIRECTORY_SEPARATOR . 'referencia/medicos' . DIRECTORY_SEPARATOR . $referencia->filename;
		
		if(file_exists($ruta)) {
			
			if(unlink($ruta)) {
				$referencia->delete();
				$res["code"] = 1;
				$res["message"] = 'El documento se elimino satisfactoriamente!';
				$res["data"] = $documento;
			  }else {
				  $res["code"] = 0;
				  $res["message"] = 'There was a error deleting the file ' . $referencia->nomDocumento;
				  $res["data"] = $documento;
			  }
		
		}else {
				  $res["code"] = 0;
				  $res["message"] = 'Archivo ' . $ruta  . ' no fue localizado';
				  $res["data"] = $documento;

		}

		return $response->getBody()->write(json_encode($res));
    }
	public function AsociarEnfermera(Request $request, Response $response, $args)
	{
		$res = ["code" => 0, "message" => "" , "data" => null];

		$id = $request->getAttribute('id');
		$id_enfermera = $request->getParam('id');

		$medico = Medico::find($id);
		$medico->enfermeras()->attach($id_enfermera); //$n = $medico->enfermeras()->save($enfermeraUpdate);

		$enfermera = Enfermera::find($id_enfermera);

		$res['code'] = 1;
		$res['message'] = 'Asocidado correctamente';
		$res['data'] = $enfermera;


		return $response->getBody()->write(json_encode($res));
	}
	public function DesasociarEnfermera(Request $request, Response $response, $args)
	{
    $res = ["code" => 0, "message" => "" , "data" => null];

    $id = $request->getAttribute('id');
    $id_enfermera = $request->getParam('id');

    $medico = Medico::find($id);
    $n = $medico->enfermeras()->detach($id_enfermera);

    $enfermera = Enfermera::find($id_enfermera);
     if($n){
        $res['code'] = 1;
        $res['message'] = 'Registro la baja con exito';
        $res['data'] = $enfermera;
      }else{
        $res['code'] = 0;
        $res['message'] = 'No se registro';
        $res['data'] = $enfermera;
      }
    return $response->getBody()->write(json_encode($res));
  }
	public function Totales(Request $request, Response $response, $args)
	{
		$id = $request->getAttribute('id');
		$medico = Medico::find($id);

		$medico->loadMissing(['casosclinicos', 'acumulados', 'canjes', 'productos']);

		$casosclinicos = $medico->casosclinicos->count();
		$productos = $medico->productos->count();
		$puntos = $medico->acumulados->sum('puntos');
		$canjes = $medico->canjes->where('status', 3)->sum('redimir');
		$saldo = $puntos - $canjes;

		$totales = ['casosclinicos' => $casosclinicos, 'productos' => $productos, 'puntos' => $puntos, 'saldo' => $saldo];
		return $response->getBody()->write(json_encode($totales));
	}
	private function correo_solicitud_canje($medico, $redimir)
	{
        $saldo = 0;
        $redimirTotal = 0;
        $puntosTotal = 0;

        foreach($medico->redimirmedico as $rd) {
                if($rd->status == 3)
                   $redimirTotal += $rd->redimir;
        }
        
		foreach($medico->productos as $producto) {
                   $puntosTotal += $producto->m_puntos;
        }
        
		$saldo = $puntosTotal - $redimirTotal;

        $descripcion = ($redimir->descripcion ?: "");
        $nombreMedico =  $medico->nombre . ' ' . $medico->primerApellido;
        $importe = $redimir->redimir;
        $fechaSolicitud = "----";
        $fecha = $redimir->fechaInicio;

        if($fecha) 
		{
          $date = date_create($fecha);
          $fechaSolicitud = date_format($date,"d/m/Y");
        }

        if($redimir->concepto == 1 )
        {
          $mensaje_medico = "
              <html>
                <head>
                  <title>SOLICITUD DE CANJE (Caso Clínico)</title>
                </head>
                <body>
                  <div class='contenedor'>
                    <div>
                      <img src='https://casosclinicos.online/imgmail/redimirpuntos.png'/>
                    </div>
                    <p>Estimado Doctor <span> $nombreMedico </span>, hemos recibido su petici&oacuten de canje de sus puntos de casosclinicos.online</p>

                    <p> Para canjear sus puntos es necesario que nos indique de que manera requiere realizarlo: </p>
                    <ol>
                    <li> Transferencia bancaria </li>
                    <li> Entrega física </li>
                    </ol>

                    <p>Transferencia bancaria</p>
                    <p>Si es por transferencia bancaria favor de enviar los siguientes datos:</p>
                    <ol>
                    <li>Nombre de institución bancaria</li>
                    <li>N&uacutemero de cuenta</li>
                    <li>Nombre del titular</li>
                    <li>Monto a depositar</li>
                    <li>Referencia de pago</li>
                    </ol>

                    <p>Entrega f&iacutesica</p>
                    <p>Si es por medio de entrega f&iacutesica favor de indicar los siguientes datos:</p>
                    <ol>
                    <li>Nombre del representante de ventas</li>
                    <li>Lugar de entrega (Hospital)</li>
                    <li>Fecha y hora de la entrega</li>
                    </ol>

                    <p>Le agradecemos la respuesta de este correo para dar inicio a su proceso de redimir sus puntos de casosclinicos.online</p>

                    <p> Saludos. </p>
                    <a href='https://casosclinicos.online'>www.casosclinicos.online</a>
                  </div>
                </body>
              </html>
              ";
        }
        
		else if ($redimir->concepto == 2)
        {
              $mensaje_medico = "
                              <html>
                              <head>
                                <title>SOLICITUD DE CANJE (Caso Clínico)</title>
                              </head>
                              <body>
                                <div class='contenedor'>
                                <div><img src='https://casosclinicos.online/imgmail/congreso.png'/></div>
                                <p>Estimado Doctor <span> $nombreMedico </span> , hemos recibido su petición de redimir sus puntos de casosclinicos.online $descripcion </p>

                                <p> Para redimir sus puntos es necesario que nos indique los siguientes datos: </p>
                                <ol>
                                  <li> Nombre completo: </li>
                                  <li> Especialidad: </li>
                                  <li> Puesto: </li>
                                  <li> ¿Es usted socio?: </li>
                                  <li> teléfono de contacto: </li>
                                </ol>

                                <p>Le agradecemos la respuesta de este correo para dar inicio a su proceso de redimir sus puntos de casosclinicos.online</p>

                                <p> Saludos. </p>
                                <a href='https://casosclinicos.online'>www.casosclinicos.online</a>
                                </div>
                              </body>
                              </html>
                              ";
        }
        
		$mensaje_alertas = "
                        <html>
                        <head>
                          <title>SOLICITUD DE CANJE DEL MEDICO (Caso Clínico)</title>
                        </head>
                        <body>
                          <div class='contenedor'>
                          <div><img src='https://casosclinicos.online/imgmail/congreso.png'/></div>
							<h1> CORREO DE PRUEBA  DATOS NO VALIDOS </h1>	
                          <p>Fecha Solicitud: $fechaSolicitud</p>
                          <p>Nombre Médico: $nombreMedico</p>
                          <p>Importe Solicitando: $importe</p>
                          <p>Saldo: $saldo</p>
                          <p>Comentario: $descripcion</p>
                          <p><a href='https://casosclinicos.online'>www.casosclinicos.online</a></p>
                          </div>
                        </body>
                        </html>
                        ";
        $para_medico = ( $medico->correo ?  $medico->correo : 'alertas_cc@biotechsa.com');

        $para_alertas = 'alertas_cc@biotechsa.com';
        $titulo = "SOLICITUD CANJE MEDICO";

        // Para enviar un correo HTML, debe establecerse la cabecera Content-type
        $cabeceras  = 'From: alertas@casosclinicos.online' . "\r\n" .   'Reply-To: alertas@casosclinicos.online' . "\r\n" .   'X-Mailer: PHP/' . " \r\n";
        $cabeceras .= 'MIME-Version: 1.0' . "\r\n";
        $cabeceras .= 'Content-type: text/html; charset=utf-8' . "\r\n";

        // Enviarlo charset=iso-8859-1
        mail($para_alertas, $titulo, $mensaje_alertas, $cabeceras);

        // Enviarlo
        mail($para_medico, $titulo, $mensaje_medico, $cabeceras);
  }

}