<?php
namespace App\Controllers;

use \Psr\Http\Message\ServerRequestInterface as Request;
use \Psr\Http\Message\ResponseInterface as Response;

use App\Models\CasoClinico;
use App\Models\Producto;
use App\Models\Batch;
use App\Models\Venta;

use App\FilesCsv\CsvHandler;
use App\Exceptions\CsvException;

use App\FilesCsv\BatchHandler;
use App\Exceptions\UpBatchException;

use Carbon\Carbon;

class ProductosController
{
  public function ReadAll(Request $request, Response $response, $args)
  {
    $productos = Producto::all();

    $productos->loadMissing(['casoclinico', 'institucion', 'medico', 'enfermera', 'venta']);

    return $response->getBody()->write(json_encode($productos));
  }
  public function Create(Request $request, Response $response, $args)
  {
      $producto_nuevo = $request->getParsedBody();
      //$codigo = $request->getParam('codigo');  $args['codigo'];
      $producto = Producto::Create($producto_nuevo);

      return $response->getBody()->write(json_encode($producto));
  }
  public function Read(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id', '0');

    $producto = Producto::find($id);
    $producto->loadMissing(['casoclinico', 'institucion', 'medico', 'enfermera']);

    return $response->getBody()->write(json_encode($producto));
  }
  public function Update(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id');

    $productoUpdate = $request->getParsedBody();
    //$nomProducto = $request->getParam('nomProducto'); $args['nomProducto'];

    $producto = Producto::find($id);
    $producto->update($productoUpdate);

    return $response->getBody()->write(json_encode($productoUpdate));
  }
  public function Delete(Request $request, Response $response, $args)
  {
    $mensaje = [];
    $id = $request->getAttribute('id');

    $producto = Producto::find($id);
    //Producto::destroy($id);

    if(!$producto->fk_casoClinico)
    {
      $producto->delete();
    //  $producto->update(['status' => 0]);
      $mensaje = [
                  "code" => 1,
                  "message" => "OK, $id producto eliminado con exito",
                  "producto" => $producto
                 ];
    }
    else
    {
      $mensaje = [
                  "code" => 0,
                  "message" => "UPS, $id producto con referencia",
                  "producto" => $producto
                  ];
    }

    return $response->getBody()->write(json_encode($mensaje));
  }
  public function Eliminar(Request $request, Response $response, $args)
  {
    $mensaje = [];
    $id = $request->getAttribute('id');

    $producto = Producto::find($id);

    if(!$producto->fk_casoClinico)
    {

      $producto->update(['status' => 0]);
      $dato1 = $producto->codigo;
      $dato2 = $producto->nomProducto;
      $mensaje = [
                  "code" => 1,
                  "message" => "OK, $dato1 producto eliminado con exito ($dato2)",
                  "producto" => $producto
                 ];
    }
    else
    {
      $mensaje = [
                  "code" => 0,
                  "message" => "UPS, $id producto con referencia",
                  "producto" => $producto
                  ];
    }

    return $response->getBody()->write(json_encode($mensaje));
  }
  public function Cancelar(Request $request, Response $response, $args)
  {
    $mensaje = [];
    $id = $request->getAttribute('id');
    $idm = $request->getAttribute('idm');

    $producto = Producto::find($id);

    if(!$producto->fk_casoClinico)
    {

      $producto->update(['status' => 0, 'tipo_mov' => $idm]);
      $mensaje = [
                  "code" => 1,
                  "message" => "OK, $producto->codigo producto Cancelado con exito",
                  "producto" => $producto
                 ];
    }
    else
    {
      $mensaje = [
                  "code" => 0,
                  "message" => "UPS, $producto->codigo producto con referencia",
                  "producto" => $producto
                  ];
    }

    return $response->getBody()->write(json_encode($mensaje));
  }
  public function CancelarCambio(Request $request, Response $response, $args)
  {
    $mensaje = [];
    $id1 = $request->getAttribute('id1');
    $id2 = $request->getAttribute('id2');

    $producto = Producto::find($id1);
    $producto_cambio = Producto::find($id2);

    $codigo_cambio = $producto_cambio->codigo;

    if(!$producto->fk_casoClinico)
    {

      $producto->update(['status' => 0, 'tipo_mov' => 2, 'codigo_change' => $codigo_cambio ]);

      $mensaje = [
                  "code" => 1,
                  "message" => "OK, $producto->codigo producto Cancelado con exito",
                  "producto" => $producto
                 ];
    }
    else
    {
      $mensaje = [
                  "code" => 0,
                  "message" => "UPS, $$producto->codigo producto con referencia",
                  "producto" => $producto
                  ];
    }

    return $response->getBody()->write(json_encode($mensaje));
  }
  public function Codigo(Request $request, Response $response, $args)
  {
    $codigo = $request->getAttribute('des');

    $producto = Producto::where('codigo', $codigo)->first();

    return $response->getBody()->write(json_encode($producto));
  }
  public function UpdatePuntosMedico(Request $request, Response $response, $args)
  {
      $res = ["code" => 0, "message" => "" , "data" => null];

      $id = $request->getAttribute('id');

      $puntos = $request->getParam('m_puntos'); // $args['codigo'];

      $producto = Producto::find($id);

      $producto->m_puntos = $puntos;

      if($producto->save()){
        $res["code"] = 1;
        $res["message"] = "Registro con exito";
        $res["data"] = $producto;
      }else{
        $res["code"] = 0;
        $res["message"] = "No se registro";
        $res["data"] = $producto;
      }

      return $response->getBody()->write(json_encode($res));
  }
  public function UpdatePuntosEnfermera(Request $request, Response $response, $args)
  {
      $res = ["code" => 0, "message" => "" , "data" => null];

      $id = $request->getAttribute('id');
      $puntos = $request->getParam('e_puntos'); // $args['codigo'];

      $producto = Producto::find($id);

      $producto->e_puntos = $puntos;

      if($producto->save()){
        $res["code"] = 1;
        $res["message"] = "Registro con exito";
        $res["data"] = $producto;
      }else{
        $res["code"] = 0;
        $res["message"] = "No se registro";
        $res["data"] = $producto;
      }

      return $response->getBody()->write(json_encode($res));
  }
  public function ProductosAsignados(Request $request, Response $response, $args)
  {

      $ids = [];

      $productos = $request->getParsedBody();

      foreach ($productos as $producto)
      {
        $p = [];

        if($producto['fk_casoClinico'])
          $p['fk_casoClinico'] = $producto['fk_casoClinico'];
        if($producto['fk_medico'])
          $p['fk_medico'] = $producto['fk_medico'];
        if($producto['m_puntos'])
          $p['m_puntos'] = $producto['m_puntos'];
        if($producto['fk_enfermera'])
          $p['fk_enfermera'] = $producto['fk_enfermera'];
        if($producto['e_puntos'])
          $p['e_puntos'] = $producto['e_puntos'];
        if($producto['fechaAplicacion'])
          $p['fechaAplicacion'] = $producto['fechaAplicacion'];

        $row = Producto::where('id', $producto['id'])->update($p);

        $ids[] = $producto['id'];
      }

      $this->Correo_Codigo($productos[0]["fk_casoClinico"]);

      $casoClinico = CasoClinico::find($productos[0]['fk_casoClinico']);
      $casoClinico->loadMissing(['productos', 'enfermera', 'estado', 'centromedico', 'institucion', 'documentos', 'seguimientos']);

      return $response->getBody()->write(json_encode($casoClinico));
  }
  public function Fecha(Request $request, Response $response, $args)
  {
    $fechaHoy = Carbon::now();

    $fecha = ['hoy' => $fechaHoy ];

    return $response->getBody()->write( json_encode($fecha) );
  }
  public function UpLote(Request $request, Response $response, $args)
  {
      $path = $_SERVER['DOCUMENT_ROOT'] . '/upload/';

      $mensaje = [];

      $csv = new CsvHandler();
      $batch = new BatchHandler();

      $files = $request->getUploadedFiles();
      $file = $files['file'];

      if( $csv->getErrorLoad($file->getError()) ){

          $media= $file->getClientMediaType();
          $tamano = $file->getSize();

          $extension = pathinfo($file->getClientFilename(), PATHINFO_EXTENSION);
          $basename = 'load' . strval(mt_rand());
          $filename = sprintf('%s.%0.8s', $basename, $extension);

          $file->moveTo($path . $filename);

          $csv->Extension($extension);
          $datos = $csv->Read($path . $filename);

          $mensaje = $batch->procesar_lote($datos, $file->getClientFilename(), $filename, $path);
      };

    return $response->getBody()->write(json_encode($mensaje));
  }
  private function Correo_Codigo($id)
  {
      $casoClinico = CasoClinico::find($id);

      $casoClinico->loadMissing(['medico', 'productos', 'enfermera']);
      $codigoslineal="";
      $codigos = "";
      $enfermera = "";

      foreach ($casoClinico->productos as $k => $producto)
      {
          $codigos .=  "<li> $producto->codigo  $producto->nomProducto</li>";
          if($codigoslineal) {
            $codigoslineal .= ', ';
          }
            $codigoslineal .=  $producto->codigo;

      }

      $para_medico = $casoClinico->medico->correo;

      $para_alertas = 'alertas_cc@biotechsa.com';

      $medico = $casoClinico->medico->nombre . " " . $casoClinico->medico->primerApellido . " " . $casoClinico->medico->primerApellido ?:'';
      $folio = $casoClinico->folio;
      $serie = $casoClinico->serie;
      $paciente = $casoClinico->paciente;
      $hospital = $casoClinico->hospital;
      $tratamiento = $casoClinico->tratamiento;
      $fechaCX = "----";

      $fecha = $casoClinico->fechaAplicacionProducto;
      if($fecha) {
        $date = date_create($fecha);
        $fechaCX = date_format($date,"d/m/Y");
      }


      if($casoClinico->enfermera)
      $enfermera =  $casoClinico->enfermera->nombre . " " . $casoClinico->enfermera->primerApellido;

      // título
      $titulo = 'CONFIRMACION DE CODIGO';
      // mensaje
      $mensaje_medico = "<html>
                                <head>
                                    <title>CONFIRMACION DE CODIGO</title>
                                </head>
                                <body>
                                      <div class='contenedor'>
                                          <div><img src='http://democasosclinicos.online/imgmail/confirmacioncodigo.jpg' /></div>
										  
										  <h1> CORREO DE PRUEBA  DATOS NO VALIDOS </h1>	
										  
                                          <p>Estimado Dr. <span> $medico </span></p>
                                          <p>Nos complace informar que el Caso Clínico No.CX $folio se asignado a el(los) producto(s)</p>
                                          <ul>$codigos</ul>
                                          <p> te invitamos a dar seguimiento a tu caso e incrementar tus puntos en cada uno. </p>
                                          <p>Saludos</p>

                                          <p><a href='http://www.democasosclinicos.online'>www.casosclinicos.online</a></p>

                                      </div>
                                </body>
                          </html>";
      // mensaje
      $mensaje_alertas = "
                        <html>
                          <head>
                            <title>CONFIRMACION DE CODIGO</title>
                          </head>
                          <body>
                            <div class='contenedor'>
                              <div>
                                <img src='http://democasosclinicos.online/imgmail/confirmacioncodigo.jpg'/>
                              </div>
							  
							  <h1> CORREO DE PRUEBA  DATOS NO VALIDOS </h1>	
							  
                              <p>Fecha CX: $fechaCX </p>
                              <p>Códigos: $codigoslineal </p>
                              <p>Médico Tratante: $medico  </p>
                              <p>Alta Caso Clínico Consecutivo: $serie </p>
                              <p>Alta Caso Clínico Médico: $folio  </p>
                              <p>Nombre Paciente: $paciente </p>
                              <p>Hospital: $hospital </p>
                              <p>Tratamiento: $tratamiento </p>
                              <p>Técnico: $enfermera </p>
                              <p><a href='http://www.democasosclinicos.online'>www.casosclinicos.online</a></p>
                            </div>
                          </body>
                        </html>
                        ";

      // Para enviar un correo HTML, debe establecerse la cabecera Content-type
      $cabeceras  = 'From: alertas@casosclinicos.online' . "\r\n" .
                    'Reply-To: alertas@casosclinicos.online' . "\r\n" .
                    'X-Mailer: PHP/' . " \r\n";
      $cabeceras .= 'MIME-Version: 1.0' . "\r\n";
      $cabeceras .= 'Content-type: text/html; charset=utf-8' . "\r\n";

      // Enviarlo charset=iso-8859-1
      mail($para_alertas, $titulo, $mensaje_alertas, $cabeceras);

      // Enviarlo
      mail($para_medico, $titulo, $mensaje_medico, $cabeceras);
  }

}
