<?php
namespace App\Controllers;

use \Psr\Http\Message\ServerRequestInterface as Request;
use \Psr\Http\Message\ResponseInterface as Response;

use App\Models\Supervisor;
use App\Models\Enfermera;
use App\Models\Usuario;
use App\Models\CasoClinico;
use App\Models\Producto;

class SupervisoresController
{
  public function ReadAll(Request $request, Response $response, $args)
  {
    $supervisores = Supervisor::all();
    $supervisores->loadMissing([ 'enfermeras', 'usuario']);

    return $response->getBody()->write(json_encode($supervisores));
  }
  public function Create(Request $request, Response $response, $args)
  {

    $supervisor_nuevo = $request->getParsedBody();
    $supervisor = Supervisor::create($supervisor_nuevo);

    return $response->getBody()->write(json_encode($supervisor));
  }
  public function Read(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id', '0');
    $supervisor = Supervisor::find($id);
    $supervisor->loadMissing([ 'enfermeras', 'usuario']);

    return $response->getBody()->write(json_encode($supervisor));
  }
  public function Update(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id', '0');
    $supervisor = Supervisor::find($id);
    $supervisor->save();

    return $response->getBody()->write(json_encode($supervisor));
  }
  public function Delete(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id', '0');
    $supervisor = Supervisor::find($id);
    $supervisor->delete();

    return $response->getBody()->write(json_encode($supervisor));
  }

  public function AsociarEnfermera(Request $request, Response $response, $args)
  {
    $res = ["code" => 0, "message" => "" , "data" => null];

    $id = $request->getAttribute('id');
    $id_enfermera = $request->getParam('id');

    $supervisor = Supervisor::find($id);
    $enfermera = Enfermera::find($id_enfermera);

   if(!$enfermera->fk_supervisor)
   {
     $enfermera->supervisor()->associate($supervisor);
     $enfermera->save();

     $res['code'] = 1;
     $res['message'] = 'Asocidado correctamente';
     $res['data'] = $enfermera;
   } else {
           $res['code'] = 0;
           $res['message'] = 'Enfermera ya esta asociada';
           $res['data'] = $enfermera;
   }

    return $response->getBody()->write(json_encode($res));
  }
  public function DesasociarEnfermera(Request $request, Response $response, $args)
  {
    $res = ["code" => 0, "message" => "" , "data" => null];

    $id = $request->getAttribute('id');
    $id_enfermera = $request->getParam('id');

    $supervisor = Supervisor::find($id);
    $enfermera = Enfermera::find($id_enfermera);
    if($enfermera->fk_supervisor)
    {
      $enfermera->supervisor()->dissociate();
      $enfermera->save();

      $res['code'] = 1;
      $res['message'] = 'Registro la baja con exito';
      $res['data'] = $enfermera;

    } else {

      $res['code'] = 0;
      $res['message'] = 'no hay relacion';
      $res['data'] = $enfermera;

    }

    return $response->getBody()->write(json_encode($res));
  }
  public function Totales(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id');
    $supervisor = Supervisor::find($id);
    $supervisor->loadMissing([ 'enfermeras', 'usuario']);

    $enfermeras = $supervisor->enfermeras->pluck('id');

    $casosClinicos = CasoClinico::whereIn('fk_enfermera', $enfermeras)->get();
    $productos = Producto::whereIn('fk_enfermera', $enfermeras)->get();

    $totales = ["casosClinicos" => $casosClinicos, "productos" => $productos ];

    return $response->getBody()->write(json_encode($totales));
  }

}
