<?php
namespace App\Controllers;

use \Psr\Http\Message\ServerRequestInterface as Request;
use \Psr\Http\Message\ResponseInterface as Response;

use App\Models\Usuario;
use App\Models\Medico;
Use App\Models\Enfermera;
Use App\Models\Supervisor;
Use App\Models\Productos;
Use App\Models\DocumentoUsuario;

use Carbon\Carbon;

class UsuariosController
{

  public function Create(Request $request, Response $response, $args)
  {
    $usuario = $request->getParsedBody();
    
	$usuario_nuevo = Usuario::Create($usuario);
	
    return $response->getBody()->write(json_encode($usuario_nuevo));
  }
  public function Read(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id', '0');

    $usuario = Usuario::find($id);
    $usuario->loadMissing(['medico', 'enfermera', 'rol', 'documentos']);

    return $response->getBody()->write(json_encode($usuario));
  }
  public function Update(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id');
    $usuario_update = $request->getParsedBody();

    $usuario = Usuario::find($id);
    $usuario->update($usuario_update);

    return $response->getBody()->write(json_encode($usuario));
  }
  public function Delete(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id');

    $usuario = Usuario::find($id);
    $usuario->delete();

    return $response->getBody()->write(json_encode($usuario));
  }
  public function Desactivar(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id');

    $usuario = Usuario::find($id);
    $usuario->update(["activo" => 0]);

    return $response->getBody()->write(json_encode($usuario));
  }
  public function Activar(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id');

    $usuario = Usuario::find($id);
    $usuario->update(["activo" => 1]);

    return $response->getBody()->write(json_encode($usuario));
  }
  public function ReadAll(Request $request, Response $response, $args)
  {
    $usuarios = Usuario::all();
    $usuarios->loadMissing(['rol']);
    return $response->getBody()->write(json_encode($usuarios));
  }
  public function UsuarioMedicoCreate(Request $request, Response $response, $args)
  {

    $data  =  $request->getParsedBody();

    $usuario = $data["usuario"];
    $medico = $data["medico"];


    $medico_nuevo = new Medico($medico);
    $usuario_nuevo = Usuario::Create($usuario);
    $usuario_nuevo->medico()->save($medico_nuevo);

  //  $this->correo_nuevo($usuario_nuevo, 'Nuevo Medico');

    return $response->getBody()->write(json_encode($usuario_nuevo));
  }
  public function UsuarioMedicoUpdate(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id', '0');
    $data  =  $request->getParsedBody();

    $usuarioUpdate = $data["usuario"];
    $medicoUpdate = $data["medico"];

    //$medicoUpdate["fechaRegistro"] = Carbon::now();

    $usuario = Usuario::find($id);
    $usuario->update($usuarioUpdate);

    $usuario->medico()->update($medicoUpdate);

    //$this->correo_nuevo($usuario, 'Medico Actualizado');

    return $response->getBody()->write(json_encode($usuario));
  }
  public function UsuarioEnfermeraCreate(Request $request, Response $response, $args)
  {
    $data  =  $request->getParsedBody();

    $usuario = $data["usuario"];
    $enfermera = $data["enfermera"];    

    $enfermera_nuevo = new Enfermera($enfermera);
    $usuario_nuevo = Usuario::Create($usuario);
    $usuario_nuevo->enfermera()->save($enfermera_nuevo);

    $this->correo_nuevo($usuario_nuevo, 'Nueva Enfermera');

    return $response->getBody()->write(json_encode($usuario_nuevo));
  }
  public function UsuarioEnfermeraUpdate(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id');
    $data = $request->getParsedBody();

    $usuarioUpdate = $data["usuario"];
    $enfermeraUpdate = $data["enfermera"];

    $usuario = Usuario::find($id);
    $usuario->update($usuarioUpdate);
    $usuario->enfermera()->update($enfermeraUpdate);

    return $response->getBody()->write(json_encode($usuario));
  }
  public function UsuarioSupervisorCreate(Request $request, Response $response, $args)
  {
    $data  =  $request->getParsedBody();

    $usuario_nuevo = $data["usuario"];
    $supervisor_nuevo = $data["supervisor"];

    $supervisor = new Supervisor($supervisor_nuevo);
    $usuario = Usuario::Create($usuario_nuevo);
    $usuario->supervisor()->save($supervisor);

  //$this->correo_nuevo($usuario, 'Nuevo Supervisor');

    return $response->getBody()->write(json_encode($usuario));
  }
  public function UsuarioSupervisorUpdate(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id', '0');
    $data  =  $request->getParsedBody();

    $usuario_update = $data["usuario"];
    $supervisor_update = $data["supervisor"];

    $usuario = Usuario::find($id);
    $usuario->update($usuario_update);

    $usuario->supervisor()->update($supervisor_update);

  //$this->correo_nuevo($usuario, 'Supervisor Actualizado');

    return $response->getBody()->write(json_encode($usuario));
  }
  public function UsuarioOperativoDelete(Request $request, Response $response, $args)
  {
    $res = ["code" => 0, "message" => "No se cumplio la funcion" , "data" => null];
    $id = $request->getAttribute('id');

    $usuario = Usuario::find($id);

    if($usuario->fk_rol == 3 ) //medicos
    {
      $medico = $usuario->medico;

      if( $medico->casosclinicos->count() == 0 && $medico->productos->count() == 0 && $medico->enfermeras->count() == 0 )
      {
        $medico->delete();
        $usuario->delete();
        $res["code"] = 1;
        $res["message"] = "eliminado correctamente";
      } else {
        $res["message"] = "No se puede eliminar este usuario por referencias de datos";
      }

    }
    else if($usuario->fk_rol == 5 ) //enfermera
    {

        $enfermera = $usuario->enfermera;
        if( $enfermera->casosclinicos->count() == 0 && $enfermera->productos->count() == 0 && $enfermera->medicos->count() == 0 && $enfermera->fk_supervisor==null )
        {
          $enfermera->delete();
          $usuario->delete();

          $res["code"] = 1;
          $res["message"] = "eliminado correctamente";
        }
        else {
          $res["message"] = "No se puede eliminar este usuario por referencias de datos";
        }

    }
    else if($usuario->fk_rol == 6)  //supervisor
    {
        $supervisor = $usuario->supervisor;

        if( $supervisor->enfermeras->count() == 0)
        {
          $supervisor->delete();
          $usuario->delete();

          $res["code"] = 1;
          $res["message"] = "eliminado correctamente";
        }
        else {
          $res["message"] = "No se puede eliminar este usuario por referencias de datos";
        }

    }
    else
    {
      $usuario->delete();
      $res["code"] = 1;
      $res["message"] = "eliminado correctamente";
    }

    $res["data"] = $usuario;
    return $response->getBody()->write(json_encode($res));
  }
  public function Documentos(Request $request, Response $response, $args)
  {
    $id = $request->getAttribute('id');

    $documentos = DocumentoUsuario::where('fk_usuario', $id)->get();

    return $response->getBody()->write(json_encode($documentos));
  }
  public function RecuperacionPassWord(Request $resquest, Response $response, $args)
  {
    $id = $request->getAttribute('id', '0');

    $usuario = Usuario::find($id);

    // enviar contraseña por correo
    //this->email_password_update($usuario);

    return $response->getBody()->write(json_encode($usuario));
  }
  public function UpdatePassword(Request $request, Response $response, $args)
  {

    $id = $request->getAttribute('id');
    $usuarioUpdate = $request->getParsedBody();

    $columns = ['password' => $usuarioUpdate['password']];

    $usuario = Usuario::find($id);
    $usuario->update($columns);

    $this->email_password_update($usuario);

    return $response->getBody()->write(json_encode($usuario));
  }
  public function email_password_update($usuario)
  {
      $apellidoPaterno = empty($usuario->primerApellido) ? '' : $usuario->primerApellido;
      $apellidoMaterno = empty($usuario->segundoApellido) ? '' : $usuario->segundoApellido;
      $nombre = $usuario->nombre . ' ' . $apellidoPaterno . ' ' . $apellidoMaterno;

      $movil = empty( $usuario->telefonoMovil ) ? '' : " <p>Telefono Movil " . " " . $usuario->telefonoMovil . "</p>" ;

      $para_usuario = $usuario->usuario;

      $mensaje_usuario = "
                          <html>
                                <head>
                                  <title>Actualizacion de Contraseña/title>
                                </head>
                                <body>
                                  <div class='contenedor'>
                                  <div><img src='http://casosclinicos.online/imgmail/bienvenido.png'/></div>
                                  <p> Hola, <span>  $nombre </span> </p>
                                  <p>la contraseña fue actualizada en CASOSCLINICOS.ONLINE</p>
                                  <p>Esta es la información de tu cuenta:</p>
                                  <p>Usuario: $usuario->usuario   </p>
                                  <p>Contraseña:  $usuario->password  </p>
                                  <p>Comienza dando clic aquí; <a href='https://www.casosclinicos.online'>www.casosclinicos.online</a></p>
                                  </div>
                                </body>
                          </html>";
                          // mensaje


                          // Para enviar un correo HTML, debe establecerse la cabecera Content-type
      $cabeceras  = 'From: alertas@casosclinicos.online' . "\r\n" .
                    'Reply-To: alertas@casosclinicos.online' . "\r\n" .
                    'X-Mailer: PHP/' . " \r\n";
      $cabeceras .= 'MIME-Version: 1.0' . "\r\n";
      $cabeceras .= 'Content-type: text/html; charset=utf-8' . "\r\n";

      // Enviarlo
      mail($para_usuario, 'Actualizacion de Contraseña', $mensaje_usuario, $cabeceras);
  }
  public function UpFile(Request $request, Response $response, $args)
  {
      $res = [
              "code" => 0,
              "message" => "No se cumplio la funcion",
              "data" => null
            ];

      $fk_usuario = $request->getParam('fk_usuario');
      $fk_documentoId = $request->getParam('fk_documentoId');
      $fechaDocumento = $request->getParam('fechaDocumento');

      $files = $request->getUploadedFiles();
      $file = $files['file'];

      if ($file->getError() === UPLOAD_ERR_OK) {

        $path = $_SERVER['DOCUMENT_ROOT'] . '/docusuarios/';

        $media = $file->getClientMediaType();

        $extension = pathinfo($file->getClientFilename(), PATHINFO_EXTENSION);
        $basename = 'docusu' . $fk_usuario . $fk_documentoId . strval(mt_rand());
        $filename = sprintf('%s.%0.8s', $basename, $extension);
        $size = $file->getSize();

        $file->moveTo( $path . $filename );

        $ruta = '/docusuarios/' . $filename;

        $documento = new DocumentoUsuario();

        $documento->fk_usuario = $fk_usuario;
        $documento->fk_documentoId = $fk_documentoId;
        $documento->nomDocumento = $filename;

        $documento->ruta = $ruta;
        $documento->media = $media;
        $documento->extension = $extension;
        $documento->tamano = $size;

        if($documento->save()){
          $res["code"] = 1;
          $res["message"] = "Registrado con exito";
          $res["data"] = $documento;
        }else{
          $res["code"] = 0;
          $res["message"] = "No se registrado";
          $res["data"] = $documento;
        }
      }
      return $response->getBody()->write(json_encode($res));
  }
  public function DeleteFile(Request $request, Response $response, $args)
  {
    $res = [
              "code" => 0,
              "message" => "No se cumplio la funcion",
              "data" => null
            ];

    $id = $request->getAttribute('id', '0');

    $documento = DocumentoUsuario::find($id);
    $documento->ruta;
    $doc = $documento;

    $ruta = dirname(__DIR__, 3) . DIRECTORY_SEPARATOR . 'docusuarios' . DIRECTORY_SEPARATOR . $documento->nomDocumento;
	
    if(file_exists($ruta)) {
        if(unlink($ruta)) {
            $documento->delete();
            $res["code"] = 1;
            $res["message"] = 'El documento se elimino satisfactoriamente!';
            $res["data"] = $doc;
        }	else {
              $res["code"] = 0;
              $res["message"] = 'There was a error deleting the file ' . $documento->nomDocumento;
              $res["data"] = $doc;
			}
    }	else {
              $res["code"] = 0;
              $res["message"] = 'Archivo ' . $ruta  . ' no fue localizado';
              $res["data"] = $doc;
		}
    return $response->getBody()->write(json_encode($res));
  }
  private function correo_nuevo($usuario, $titulo)
  {
      $apellidoPaterno = empty($usuario->primerApellido) ? '' : $usuario->primerApellido;
      $apellidoMaterno = empty($usuario->segundoApellido) ? '' : $usuario->segundoApellido;
      $nombre = $usuario->nombre . ' ' . $apellidoPaterno . ' ' . $apellidoMaterno;

      $movil = empty( $usuario->telefonoMovil ) ? '' : " <p>Telefono Movil " . " " . $usuario->telefonoMovil . "</p>" ;

      $para_alertas = 'alertas_cc@biotechsa.com';
      $para_usuario = $usuario->usuario;

      $mensaje_usuario = "
                          <html>
                                <head>
                                  <title>BIENVENIDO NUEVO USUARIO</title>
                                </head>
                                <body>
                                  <div class='contenedor'>
                                  <div><img src='http://casosclinicos.online/imgmail/bienvenido.png'/></div>
                                  <p> Hola, <span>  $nombre </span> </p>
                                  <p>Gracias por unirte a CASOSCLINICOS.ONLINE</p>
                                  <p>Esta es la información de tu cuenta:</p>
                                  <p>Usuario: $usuario->usuario   </p>
                                  <p>Contraseña:  $usuario->password  </p>
                                  <p>Comienza dando clic aquí; <a href='https://www.casosclinicos.online'>www.casosclinicos.online</a></p>
                                  </div>
                                </body>
                          </html>";
                          // mensaje
      $mensaje_alertas = "
                          <html>
                          <head>
                            <title>BIENVENIDO NUEVO USUARIO( $titulo )</title>
                          </head>
                          <body>
                            <div class='contenedor'>
                            <div><img src='http://casosclinicos.online/imgmail/bienvenido.png'/></div>
                            <p>Bienvenida a <span> $nombre </span> </p>
                            <p>Se unio a CASOSCLINICOS.ONLINE</p>
                            <p>Esta es la información de la cuenta:</p>
                            <p>Usuario: $usuario->usuario   </p>
                            <p>Contraseña:  $usuario->password  </p>
                            <p> $movil </p>
                            </div>
                          </body>
                          </html>
                        ";

                          // Para enviar un correo HTML, debe establecerse la cabecera Content-type
      $cabeceras  = 'From: alertas@casosclinicos.online' . "\r\n" .
                    'Reply-To: alertas@casosclinicos.online' . "\r\n" .
                    'X-Mailer: PHP/' . " \r\n";
      $cabeceras .= 'MIME-Version: 1.0' . "\r\n";
      $cabeceras .= 'Content-type: text/html; charset=utf-8' . "\r\n";

      // Enviarlo charset=iso-8859-1
      mail($para_alertas, $titulo, $mensaje_alertas, $cabeceras);

      // Enviarlo
      mail($para_usuario, $titulo, $mensaje_usuario, $cabeceras);
  } 	
}
