<?php
namespace App\FilesCsv;

use Illuminate\Support\Facades\DB;
use App\Exceptions\UpBatchException;
use App\Exceptions\VentaException;
use App\Exceptions\ProductoException;

use Illuminate\Support\Collection;

use App\Models\Producto;
use App\Models\Batch;
use App\Models\Venta;

use Carbon\Carbon;
use DateTime;

class BatchHandler
{

  public function procesar_lote($datos, $archivo_fuente, $archivo_destino, $path)
  {
          $productos_org = array();
          $batchs_org = array();
          $ventas_org = array();

          $row_buff = array();
          foreach($datos as $row)
          {
            if(empty($row_buff))
            {
              $row_buff = $row;
              $batchs_org[] = [
                        "noBatch" => $row[0],
                        "fecha" => $row[1],
                        "noproductos" => 0,
                        "archivo_fuente" => $archivo_fuente,
                        "archivo_destino" => $archivo_destino,
                        "ruta" => $path,
                        "created_at" => Carbon::now(),
                        "updated_at" => Carbon::now()
                      ];
              $ventas_org[] = [
                        "noBatch" => $row[0],
                        "fecha" => $row[1],
                        "noVenta_noReferencia" => $row[2],
                        "noproductos" => 0,
                        "fk_institucion" => $row[3],
                        "tipo" => $row[8],
                        "created_at" => Carbon::now(),
                        "updated_at" => Carbon::now()
                      ];
              $productos_org[] = [
                              "noBatch" => $row[0],
                              "codigo" => $row[4],
                              "nomProducto" => $row[5],
                              "lote" => $row[6],
                              "caducidad" => $row[7],
                              "created_at" => Carbon::now(),
                              "updated_at" => Carbon::now()      // date("Y-m-d H:i:s")  2001-03-10 17:16:18 (el formato DATETIME de MySQL)
                            ];
            }
            else
            {
              if( $row_buff[0]==$row[0] ) //el mismo batch
              {
                if( $row_buff[2]==$row[2] ){
                      $productos_org[] = [
                                      "noBatch" => $row[0],
                                      "codigo" => $row[4],
                                      "nomProducto" => $row[5],
                                      "lote" => $row[6],
                                      "caducidad" => $row[7],
                                      "created_at" => Carbon::now(),
                                      "updated_at" => Carbon::now()      // date("Y-m-d H:i:s")  2001-03-10 17:16:18 (el formato DATETIME de MySQL)
                                    ];
                }
                else {
                  	throw new UpBatchException("verifica el archivo", 2, array("", 2, "no es la misma factura") );
                }

              }
              else
              {
                $row_buff = $row;
                $batchs_org[] = [
                          "noBatch" => $row[0],
                          "fecha" => $row[1],
                          "noproductos" => 0,
                          "archivo_fuente" => $archivo_fuente,
                          "archivo_destino" => $archivo_destino,
                          "ruta" => $path,
                          "created_at" => Carbon::now(),
                          "updated_at" => Carbon::now()
                        ];

                $ventas_org[] = [
                          "noBatch" => $row[0],
                          "fecha" => $row[1],
                          "noVenta_noReferencia" => $row[2],
                          "noproductos" => 0,
                          "fk_institucion" => $row[3],
                          "tipo" => $row[8],
                          "created_at" => Carbon::now(),
                          "updated_at" => Carbon::now()
                        ]; //tipo = $row[8] el campo 8 se define como tipo de documento

                $productos_org[] = [
                                "noBatch" => $row[0],
                                "codigo" => $row[4],
                                "nomProducto" => $row[5],
                                "lote" => $row[6],
                                "caducidad" => $row[7],
                                "created_at" => Carbon::now(),
                                "updated_at" => Carbon::now()      // date("Y-m-d H:i:s")  2001-03-10 17:16:18 (el formato DATETIME de MySQL)
                              ];
              }

            }

          }

          $coleccion_productos = collect($productos_org);
          $grouped_productos = $coleccion_productos->groupBy('noBatch');
          $productos_grouped = $grouped_productos->all();

          $nbatchs = array();
          foreach($batchs_org as $batch)
          {
             $noBatch = $batch['noBatch'];
             $nbatchs[] = $batch['noBatch'];
             $noproductos = $productos_grouped[$noBatch]->count();

             $batch['noproductos'] = $noproductos;
             $venta_loc = array();
             foreach($ventas_org as $venta)
             {
                 if($venta['noBatch'] == $noBatch)
                 {
                  $venta['noproductos'] = $noproductos;
                  $venta_loc = $venta;
                 }
             }

             $nva_venta = new Venta($venta_loc);

             $nvo_batch = Batch::Create($batch);
             $nvo_batch->venta()->save($nva_venta);
             $nva_venta->productos()->createMany($productos_grouped[$noBatch]->all());

          }

          $productos_query = Producto::whereIn("noBatch", $nbatchs )->get();
          $no = $productos_query->count();

          return [
                  "code" => 0,
                  "message" => "OK, $no producto(s) registrado(s) con exito",
                  "productos" => $productos_query
                 ];

  }

  public function procesar_lote_old($buffer, $archivo_fuente, $archivo_destino, $path)
  {
    $patron_fecha = '/^20[0-2][0-9]\-(0?[1-9]|1[012])\-(0?[1-9]|[12][0-9]|3[01])$/';
    $patron_institucion = '/\A\d{1,2}\Z/';
    $patron_tipo  = '/\A[1-4]\Z/';
    $padron_batch = '/\d+/';
    $row_valid   = true;
    $fecha_valid = true;
    $batch_valid = true;
    $venta_valid = true;
    $tipo_valid  = true;
    $institucion_valid = true;
    $noRows = 0;
    $batch_keys = array("noBatch", "fecha", "noproductos", "archivo_fuente", "archivo_destino", "ruta", "created_at", "updated_at");
    $venta_keys = array( "noBatch", "fecha", "noVenta_noReferencia", "fk_institucion", "tipo", "noproductos", "created_at", "updated_at" );
    $producto_keys = array( "noBatch", "codigo", "nomProducto", "lote", "caducidad", "created_at", "updated_at");

    $batch_row;
    $venta_row;
    $productos_rows = array();
    $index_registro_batch=0;
    if( !empty( $buffer ) ){

        $noRows = count($buffer);
        $noBatch;
        $noVenta;
        foreach ($buffer as $row) {

          if( count($row) === 9 ) {
                list($numBatch, $fec, $numVenta) = $row;
                if( empty($noBatch) ) {
                    $noBatch = $numBatch;
                } else {
                        if(!($noBatch === $numBatch)) {
                            $batch_valid = false;
                            $index_real = $index_registro_batch + 1;
                            throw new UpBatchException("Numero de Batch no valido en el registro $index_real", 2, array("", 1, "Batch "));
                          }
                 }
                if( empty($noVenta)){
                  $noVenta = $numVenta;
                } else {
                    if( !($noVenta === $numVenta) ){
                      $venta_valid = false;
                      $index_real = $index_registro_batch + 1;
                      throw new VentaException("Numero de Venta no valido en el registro $index_real ", 2, array("", 1, "Venta "));
                    }
                }
          }
          else {
                    $row_valid = false;
                    $index_real = $index_registro_batch + 1;
                  throw new VentaException("Registro $index_real incompleto", 2, array("", 1, "Venta "));
            }
            $index_registro_batch++;
        }

        if($row_valid && $batch_valid && $venta_valid) {

          $item = current($buffer);
          list($gbatch, $gfecha, $gventa, $ginstitucion, $gcodigo, $gproducto, $glote, $gcaducidad, $gtipo) = $item;

          $batch_values = array($gbatch, $gfecha, $noRows, $archivo_fuente, $archivo_destino, $path, Carbon::now(), Carbon::now());
          $batch_row = array_combine($batch_keys, $batch_values);

          $venta_values = array($gbatch, $gfecha, $gventa, $ginstitucion,  $gtipo, $noRows, Carbon::now(), Carbon::now() );
          $venta_row = array_combine($venta_keys, $venta_values);
          $index_row=0;
          foreach ($buffer as list($batch, $fecha, $venta, $institucion, $codigo, $producto, $lote, $caducidad, $tipo)) {
            if( !preg_match($patron_fecha, $fecha, $match) ) {
              $fecha_valid = false;
              throw new ProductoException("fecha No Valido $fecha", 2, array("", 1, "La fecha no es valida "));
            } else if( !$this->validateDate($fecha) ) {
                    $fecha_valid = false;
                    throw new ProductoException("fecha No Valido $fecha en el registro $index_row", 2, array("", 1, "La fecha no es valida "));
                  }

            if( !preg_match($patron_fecha, $caducidad, $match) ) {
              $fecha_valid = false;
              throw new ProductoException("fechade caducidad No Valido $caducidad en el registro $index_row", 2, array("", 1, "La caducidad no es valida "));
            } else if( !$this->validateDate($caducidad) ) {
                    $fecha_valid = false;
                    throw new ProductoException("fecha caducidad No Valido $caducidad en el registro $index_row", 2, array("", 1, "La caducidad no es valida "));
                  }

            if( !preg_match($patron_institucion, $institucion) ) {
              $institucion_valid = false;
              throw new ProductoException("No valido institucion en el registro $index_row", 2, array("", 1, "La Institucion no es valida "));
            } // condicion de congunto de instituciones

            if( !preg_match($patron_tipo, $tipo) ) {
              $tipo_valid=false;
              throw new ProductoException("No valido tipo en el registro $index_row", 2, array("", 1, "El tipo no es valido "));
            }// condicion de congunto de tipo de venta

            if( !preg_match($padron_batch, $batch)) {
              throw new UpBatchException("No valido batch en el registro $index_row", 2, array("", 1, "Batch"));
            }

            $producto_values = array($batch,  $codigo, $producto, $lote, $caducidad, Carbon::now(), Carbon::now());
            $producto_row = array_combine($producto_keys, $producto_values);
            $productos_rows[] = $producto_row;
            $index_row++;
          // echo "Batch: $batch; fecha: $fecha; venta: $venta; institucion: $institucion; codigo: $codigo; producto: $producto; lote: $lote, tipo: $tipo -> $nota\n";

          }

DB::transaction(function(){

  $venta = new Venta($venta_row);

  $batch = Batch::Create($batch_row);
  $batch->venta()->save($venta);
  $venta->productos()->createMany($productos_rows);
});
/*

*/

        }
    }else {
      throw new UpBatchException("Batch vacio", 2, array("", 1, "Batch "));
    }

    return [
            "code" => 0,
            "message" => "OK, $noRows producto(s) registrado(s) con exito",
            "productos" => $productos_rows
           ];

  }

  protected function validateDate($date, $format = 'Y-m-d')
  {
    $d = DateTime::createFromFormat($format, $date);
    return $d && $d->format($format) == $date;
  }
}
